/* in ja_JP.UTF-8
* in public domain
*/

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import jp.sfjp.mikutoga.bin.parser.MmdFormatException;
import jp.sourceforge.mmd.ik_solver.IKSolver;
import jp.sourceforge.mmd.motion.Motion;
import jp.sourceforge.mmd.motion.Pose;
import jp.sourceforge.mmd.motion.model.Bone;
import jp.sourceforge.mmd.motion.model.Model;

/**
 * IK Solver Sample
 * @see http://mmdmotion-java.osdn.jp/wiki/MMDIKSolver
 */
public class SurfaceDrawingTest {
    private IKSolver iksolver;
    private Motion targetMotion;
    private Motion ikMotion;
    private Model targetModel;
    private Model ikModel;

    static public void main(String [] argv) {
        SurfaceDrawingTest test=null;
        System.out.println("Model Loading..");
        try{
            test=new SurfaceDrawingTest();
        }catch(FileNotFoundException e){
            System.err.println("ファイルが見つかりません: "+e);
        }catch(IOException e){
            System.err.println("IOエラー: "+e);
        }catch(MmdFormatException e){
            System.err.println("MMD ファイルとして異常: "+e);
        }
        if(test==null)
            System.exit(-1);
        System.out.println("All loaded!");
        test.setBones();
        test.solve();
    }

    public SurfaceDrawingTest() throws FileNotFoundException, IOException, MmdFormatException{

        FileInputStream fis=new FileInputStream("test/教室.pmd");
        targetModel=Model.fromPMD(fis);
        fis.close();
        System.out.println("Model: "+targetModel.getName()+" Done!");

        fis=new FileInputStream("test/鼓カノン.pmd");
        ikModel=Model.fromPMD(fis);
        fis.close();
        System.out.println("Model: "+ikModel.getName()+" Done!");

        fis=new FileInputStream("test/target.vmd");
        targetMotion=new Motion().fromVMD(fis);
        fis.close();

        iksolver=new IKSolver();
        fis=new FileInputStream("test/kanon.vmd");
        ikMotion=new Motion().fromVMD(fis);
        fis.close();
    }

    private void setBones(){
        String [] nameOfBones=new String[] {"左人指３先","左人指１","左手首","左ひじ","左腕","左肩","上半身"};
        Bone [] bones=new Bone[nameOfBones.length];

        for(int i=0;i<nameOfBones.length;i++){
            bones[i]=ikModel.get(nameOfBones[i]);
        }
        iksolver.setBones(bones);

        iksolver.setLimits(1,IKSolver.RLIMIT_NOX|IKSolver.RLIMIT_NOY);
        iksolver.setLimits(3,IKSolver.RLIMIT_NOX|IKSolver.RLIMIT_NOZ);
        iksolver.setLimits(5,IKSolver.RLIMIT_NOX|IKSolver.RLIMIT_NOY);
    }

    public void solve(){
        Bone target=targetModel.get("右黒板DB3");
        Pose [] poses=targetMotion.get(target.getName());
        int frame;
        double distance;
        Motion results=new Motion(ikMotion.getModelName());

        // test1 ここから
        for(Pose p:poses){
            frame=p.frame;
        //test1 ここまで
/*
        // test2ここから
        int end=targetMotion.getMaxFrame();
        for(frame=0;frame<end;frame++){
        //test 2ここまで
*/

            // そのフレームの状態にする。
            ikModel.setPoses(ikMotion.getInterporate(frame));
            targetModel.setPoses(targetMotion.getInterporate(frame));

            // 変更フラグをリセットして変更したボーンだけを記録する.
            ikModel.resetChanged();
            distance=iksolver.solve(target.getPos());
            System.out.println("frame: "+frame+" distance: "+distance);
            results.putAll(ikModel.getChanged(), frame);
        }

        FileOutputStream fos=null;
        try {
            fos = new FileOutputStream("test/solution.vmd");
            results.toVMD(fos);
            fos.close();
        } catch (FileNotFoundException ex) {
            System.err.println("ファイルが見つかりません: "+ ex);
        } catch (IOException ex) {
            System.err.println("ファイルが書けません: "+ ex);
        }
    }
}
